import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import ir.huri.jcal.JalaliCalendar;

public class Voting {
	// type of question
	private int type ;
	// question for voting
	private String question ;
	//arrayList of persons who voted
	private ArrayList<Person> voters ;
	//HashMap of choices and votes
	private HashMap<String, HashSet<Vote>> listOfVotesToChoices ; ;
	/**
	 * Create new voting with given type and question
	 * @param type Type of vote
	 * @param question
	 */
	public Voting(int type, String question) {
		voters = new ArrayList<Person>() ;
		listOfVotesToChoices = new HashMap<String, HashSet<Vote>>() ;
		this.type = type;
		this.question = question;
	}
	/**
	 * Get a question of voting
	 * @return
	 */
	public String getQuestion() {
		return question ;
	}
	/**
	 * Create a choice for question
	 * @param option choice for question
	 */
	public void createChoice (String option) {
		listOfVotesToChoices.put(option, new HashSet<Vote>()) ;
	}
	
	/**
	 * Create a vote with given person and arrayList of choices
	 * @param person Person who voted
	 * @param choice Choices of person for question
	 */
	public void createVote(Person person , ArrayList<String> choice) {
		String date = new JalaliCalendar().toString() ;
		if(type== 0) {
			if(listOfVotesToChoices.containsKey(choice.get(0)) ) {
				Vote vote = new Vote(person, date) ;
				boolean bool = listOfVotesToChoices.get(choice.get(0)).contains(vote) ;
				if(bool) {
					System.out.println("Votes have alreadey been submitted!\n");
				}
				else {
					listOfVotesToChoices.get(choice.get(0)).add(vote) ;
					voters.add(person) ;
				}
			}
		}
		else {
			boolean voterBool = false ;
			boolean boool = false ;
			for(String str : choice) {
				if(listOfVotesToChoices.containsKey(str)) {
					boool = true ;
					Vote vote = new Vote(person, date) ;
					boolean bool = listOfVotesToChoices.get(str).contains(vote) ;
					if(!bool){
						listOfVotesToChoices.get(str).add(vote) ;
						voterBool = true ;
					}
				}
			}
			if(voterBool) {
				voters.add(person) ;
			}
			if(!voterBool && boool ) {
				System.out.println("Votes have alreadey been submitted!\n") ;
			}
		}
	}
	/**
	 * print result of this question
	 */
	public void printVotes() {
		Set<String> keySet = listOfVotesToChoices.keySet() ;
		System.out.println("Result of this question!: " + getQuestion() + "");
		int i = 1 ;
		for(String key : keySet) {
			int choiceSize = listOfVotesToChoices.get(key).size() ;
			System.out.println("   "+ i++ + ": " + key +" --> " + choiceSize );
		}
		System.out.println("Voters : ");
		getVoters();
		System.out.println();
	}
	/**
	 * print voters who voted this question
	 */
	public void getVoters() {
		if(voters.size() == 0)
			System.out.println("   Nobody has voted this question!");
		else {
			HashSet<Vote> votes = new HashSet<Vote>() ;
			Set<String> keys = listOfVotesToChoices.keySet() ;
			for(String key : keys) {
				Set<Vote> hashSets = listOfVotesToChoices.get(key) ; 
				for(Vote hashSet : hashSets) {
					votes.add(hashSet) ;
				}
			}
			for(Vote vote : votes) {
				System.out.println("	" + vote.getPerson().toString() + " Date-->" + vote.getDate().toString() ) ;
			}
		}

	}
	/**
	 * print choices for this question
	 */
	public void getChoices() {
		int i = 1 ;
		Set<String> choices = listOfVotesToChoices.keySet() ;
		for(String choice : choices ) {
			System.out.println("   " + i++ + ": " + choice);
		}
		System.out.println();
	}
}
